package edu.kit.ipd.descartes.adaptation.evaluation;

import org.apache.log4j.Logger;

import edu.kit.ipd.descartes.adaptation.evaluation.weightingfunction.IWeightingFunction;
import edu.kit.ipd.descartes.mm.adaptation.WeightedTactic;
import edu.kit.ipd.descartes.mm.adaptation.WeightingFunction;

/**
 * Evaluates a given {@link WeightedTactic} using the weighted tactic's {@link WeightingFunction}.
 * 
 * @author nhuber
 * 
 */
public class WeightingFunctionEvaluator implements IEvaluator {

    private IWeightingFunction calculator = null;
    private static Logger logger = Logger.getLogger(WeightingFunctionEvaluator.class);

    /*
     * (non-Javadoc)
     * 
     * @see edu.kit.ipd.descartes.adaptation.evaluation.IEvaluator#evaluate()
     */
    @Override
    public void evaluate(WeightedTactic tactic) {
        WeightingFunction f = tactic.getParentStrategy().getWeightingFunction();
        
        final String functionClassName = f.getWeightingFunctionImplementation();

        Object o = null;
        try {
            o = Class.forName(functionClassName).newInstance();
        } catch (final ClassNotFoundException e) {
            e.printStackTrace();
        } catch (final InstantiationException e) {
            e.printStackTrace();
        } catch (final IllegalAccessException e) {
            e.printStackTrace();
        }

        if (!IWeightingFunction.class.isInstance(o)) {
            throw new RuntimeException("Expected a class implementing " + IWeightingFunction.class.getName());
        }
        
        calculator = (IWeightingFunction) o;
        calculator.setWeightingFunction(f);
        double newWeight = calculator.calculateWeight(tactic);
        logger.debug("Setting new weight of " + tactic.getName() + " to " + newWeight);
        tactic.setCurrentWeight(newWeight);
    }
}
